﻿// © Copyright 2011 Agilent Technologies, Inc.

using System;
using System.AddIn;
using System.Collections.Generic;
using System.Text;
using System.Windows.Forms.Integration;
using Agilent.Vee.EvoExample.ExpressionGraph;
using Agilent.Vee.Extensibility;

namespace Agilent.Vee.EvoExample
{
    /// <summary>
    /// An Extensible VEE Object (EVO) that accept a Y-Expression, then generate waveform data points.
    /// </summary>
    [AddIn("Agilent VEE EVO Sample", Description = "An EVO sample for Agilent VEE.",
    Publisher = "Agilent VEE", Version = "1.0.0.0")]
    public class ExpressionGraphEvo : IExtensibleVeeObject
    {
        GraphControl m_graph;
        List<EvoTerminal> m_inputTerminals;
        List<EvoTerminal> m_outputTerminals;
        EventHandler m_evoTerminalChanged;
        EventHandler m_notifyChanges;
        String m_error;
        IApplication m_vee;

        public ExpressionGraphEvo()
        {
            m_graph = new GraphControl();
            m_graph.ContentChanged += new EventHandler(m_graph_ContentChanged);
            m_graph.TerminalChanged += new EventHandler(m_graph_TerminalChanged);
            
            SetupOutputTerminal();
        }

        #region IExtensibleVeeObject Members        

        /// <summary>
        /// Gets the list of EVO Control input terminals. It served as the optional Data inputs terminal. 
        /// Its behavior is same with control input terminals of the orginal VEE objects. 
        /// It is recommended to avoid using Control input terminals if possible.
        /// </summary>
        public IList<EvoTerminal> ControlInputs
        {
            get { return null; }
        }

        /// <summary>
        /// Gets the list of EVO Data input terminals.
        /// </summary>
        public IList<EvoTerminal> DataInputs
        {
            get { return m_inputTerminals; }
        }

        /// <summary>
        /// Gets the list of EVO Data Output terminals.
        /// </summary>
        public IList<EvoTerminal> DataOutputs
        {
            get { return m_outputTerminals; }
        }

        /// <summary>
        /// Gets the default height of the EVO.
        /// </summary>
        public Int32 DefaultHeight
        {
            get { return 300; }
        }

        /// <summary>
        /// Gets the default height of the EVO.
        /// </summary>
        public Int32 DefaultWidth
        {
            get { return 300; }
        }

        /// <summary>
        /// Gets/Sets EVO error message. If EVO Error output terminal is added, 
        /// it will be stored in that terminal; otherwise VEE will display the 
        /// message in runtime error dialog.
        /// </summary>
        public string ErrorOutput
        {
            get
            {
                return m_error;
            }
            set
            {
                m_error = value;
            }
        }

        /// <summary>
        /// This method will be executed before the VEE program start to run.
        /// </summary>
        public void PreRun()
        {
            m_error = String.Empty;
            m_graph.HideTerminalColumn();            
        }

        /// <summary>
        /// This method will be executed when the VEE program execute to this EVO. 
        /// If there is unhandled exception in this method, 
        /// VEE will catch it and treat it as runtime error.
        /// </summary>
        public void Execute()
        {
            try
            {
                SetupInputValue();
                m_graph.CheckValue();
                m_graph.GenerateWaveform();
                m_outputTerminals[0].Value = m_graph.XCoordinate;
                m_outputTerminals[1].Value = m_graph.YCoordinate;
            }
            catch (Exception ex)
            {
                m_error = ex.Message;
            }
        }

        /// <summary>
        /// This method will be executed right after the Execute method, 
        /// and before VEE program ended.
        /// </summary>
        public void PostRun()
        {
            if (m_vee.ProductEdition != ProductEdition.VeeRun)
            {
                m_graph.ShowTerminalColumn();
            }            
        }

        /// <summary>
        /// The UI control of the Extensible VEE Object, it can be 
        /// Windows Forms Control or WPF window. Return null if there is no EVO UI.
        /// </summary>
        /// <returns>EVO UI control, null if there is no UI.</returns>
        public System.Windows.Forms.Control GetControl()
        {
            return new ElementHost() { Child = m_graph };
        }

        /// <summary>
        /// Gets the data representing the EVO state. 
        /// VEE will save the data into VEE program file.
        /// </summary>
        /// <returns>EVO state.</returns>
        public string GetObjectData()
        {
            // This string will be saved into VEE file
            DataState state = new DataState();
            state.YExpression = m_graph.YExpression;
            state.XFrom = m_graph.XFrom;
            state.XTo = m_graph.XTo;
            state.DataSize = m_graph.DataSize;
            state.YTermAdded = m_graph.YTermAdded;
            state.XTermAdded = m_graph.XTermAdded;
            state.SizeTermAdded = m_graph.DataSizeTermAdded;

            return state.ConstructState();
        }

        /// <summary>
        /// When opening a saved VEE program, saved EVO state data will be serialized into string. 
        /// The string data will be passed to EVO for EVO to process the state.
        /// </summary>
        /// <param name="data"></param>
        public void SetObjectData(string data)
        {
            // When open a saved VEE program, the saved EVO data will be pass to 
            // EVO through this.
            DataState state = new DataState();
            Boolean updated;

            state.SplitState(data, out updated);
            if (!updated)
            {
                return;
            }

            m_graph.YExpression = state.YExpression;
            m_graph.XFrom = state.XFrom;
            m_graph.XTo = state.XTo;
            m_graph.DataSize = state.DataSize;
            // Initialize terminal status.
            if (state.YTermAdded)
            {
                m_graph.YTermAdded = true;
            }

            if (state.XTermAdded)
            {
                m_graph.XTermAdded = true;
            }

            if (state.SizeTermAdded)
            {
                m_graph.DataSizeTermAdded = true;
            }
        }

        /// <summary>
        /// Launch the help of the EVO.
        /// </summary>
        public void ShowHelp()
        {
            // EVO developer may choose to show their help file
            // Or leave it blank if not needed.
            // In this example, we will prompt a dialog box to display
            // some help text  
            StringBuilder sb = new StringBuilder(256);
            sb.Append("This is an EVO example. ");
            sb.Append("This Extensible VEE Object (EVO) accept a Y-Expression, ");
            sb.Append("then generate waveform data points.");
            System.Windows.MessageBox.Show(sb.ToString(), "EVO Help", 
                System.Windows.MessageBoxButton.OK, System.Windows.MessageBoxImage.Information);
        }

        /// <summary>
        /// Gets the Default title of EVO. User can change it in VEE.
        /// </summary>
        public string Title
        {
            get { return "Expression Graph"; }
        }

        /// <summary>
        /// An event to notify VEE there is EVO changes. When this event is fired, 
        /// VEE will display an asterisk in program title bar.
        /// </summary>
        public event EventHandler ModifiedSet
        {
            add
            {
                m_notifyChanges += value;
            }
            remove
            {
                m_notifyChanges -= value;
            }
        }

        /// <summary>
        /// An event to notify VEE there is input/output/control terminal changed.
        /// </summary>
        public event EventHandler TerminalChanged
        {
            add
            {
                m_evoTerminalChanged += value;
            }
            remove
            {
                m_evoTerminalChanged -= value;
            }
        }

        /// <summary>
        /// Gets a reference to current instance of Agilent VEE application, 
        /// which is the root object of the Agilent VEE automation model. 
        /// It is good practice to check if application is not null before using it.
        /// </summary>
        /// <param name="application"></param>
        public void GetCurrentApplication(IApplication application)
        {
            m_vee = application;
        }

        /// <summary>
        /// This method will be executed after the EVO input terminal is added.
        /// </summary>
        /// <param name="name">The added EVO Input terminal name.</param>
        public void InputEvoTerminalAdded(string name)
        {            
        }

        /// <summary>
        /// This method will be executed after the EVO input terminal is removed.
        /// </summary>
        /// <param name="name">The removed EVO Input terminal name.</param>
        public void InputEvoTerminalRemoved(string name)
        {            
        }

        /// <summary>
        /// This method will be executed after the EVO control terminal is added.
        /// </summary>
        /// <param name="name">The added EVO Control input terminal name.</param>
        public void ControlEvoTerminalAdded(string name)
        {            
        }

        /// <summary>
        /// This method will be executed after the EVO control terminal is removed.
        /// </summary>
        /// <param name="name">The removed EVO Control input terminal name.</param>
        public void ControlEvoTerminalRemoved(string name)
        {            
        }      

        #endregion

        private void FireModifySet()
        {
            if (m_notifyChanges != null)
            {
                m_notifyChanges(this, EventArgs.Empty);
            }
        }

        private void FireTerminalChanged()
        {
            if (m_evoTerminalChanged != null)
            {
                m_evoTerminalChanged(this, EventArgs.Empty);
            }
        }

        private void SetupOutputTerminal()
        {
            m_outputTerminals = new List<EvoTerminal>(2);
            EvoTerminal terminal = new EvoTerminal("XCoordinate", typeof(Double[]));
            m_outputTerminals.Add(terminal);
            terminal = new EvoTerminal("YCoordinate", typeof(Double[]));
            m_outputTerminals.Add(terminal);
        }

        private void SetupInputValue()
        {
            Int32 index = 0;
            if (m_graph.YTermAdded)
            {
                m_graph.YExpression = Convert.ToString(m_inputTerminals[index].Value);
                index++;
            }

            if (m_graph.XTermAdded)
            {
                m_graph.XFrom = Convert.ToDouble(m_inputTerminals[index].Value);
                index++;
                m_graph.XTo = Convert.ToDouble(m_inputTerminals[index].Value);
                index++; ;
            }

            if (m_graph.DataSizeTermAdded)
            {
                m_graph.DataSize = Convert.ToInt32(m_inputTerminals[index].Value);
            }
        }

        void m_graph_TerminalChanged(object sender, EventArgs e)
        {
            m_inputTerminals = new List<EvoTerminal>();
            EvoTerminal term;

            if (m_graph.YTermAdded)
            {
                term = new EvoTerminal("YExpression", typeof(String));
                m_inputTerminals.Add(term);
            }

            if (m_graph.XTermAdded)
            {
                term = new EvoTerminal("XFrom", typeof(Double));
                m_inputTerminals.Add(term);
                term = new EvoTerminal("XTo", typeof(Double));
                m_inputTerminals.Add(term);
            }

            if (m_graph.DataSizeTermAdded)
            {
                term = new EvoTerminal("NumberOfPoint", typeof(Int32));
                m_inputTerminals.Add(term);
            }

            FireTerminalChanged();
        }

        void m_graph_ContentChanged(object sender, EventArgs e)
        {
            FireModifySet();
        }
    }
}
